/**************************************************************************
*
*	Vec.h
*
*	Contains a vector class
*
*	Author: Saxon Druce
*
*	Copyright (c) 2003 - 2004 Block Software Pty Ltd
*
**************************************************************************/

#ifndef _VEC_H
#define _VEC_H

/*************************************************************************/
// Include files
/*************************************************************************/

#include <math.h>

/*************************************************************************/
// Defines
/*************************************************************************/

#define M_PI	3.14159265

/*************************************************************************/
// Public types
/*************************************************************************/

// Vec_t class - holds a 2D vector
class Vec_t
{
public:

	// the x and y values
	double x, y;

	// Constructors
	Vec_t() { x = y = 0; }
	Vec_t(const double& x, const double& y) { this->x = x; this->y = y; }

	// GetAngle() - Returns the angle the vector makes with the x axis, 
	// in the range -pi to +pi, with positive clockwise.
	// ie: -pi (at -x) to -pi/2 (-y) to 0 (+x) to +pi/2 (+y) to +pi (-x).
	double GetAngle() const
	{
		return atan2(y, x);
	}

	// SetFromAngle() - Sets to a unit vector which makes the specified 
	// angle with the x axis. 
	// a.SetFromAngle(b.GetAngle()) would make a equal to b normalised 
	// (within numerical error).
	void SetFromAngle(const double& alpha)
	{
		x = cos(alpha);
		y = sin(alpha);
	}

	// multiplication by a scalar
	Vec_t operator*(double f) const { return Vec_t(x * f, y * f); }

	// addition
	Vec_t operator+(const Vec_t& rhs) const { return Vec_t(x + rhs.x, y + rhs.y); }

	// subtraction
	Vec_t operator-(const Vec_t& rhs) const { return Vec_t(x - rhs.x, y - rhs.y); }

	// in-place addition
	void operator+=(const Vec_t& rhs) { x += rhs.x; y += rhs.y; }

	// in-place multiplication by a scalar
	void operator*=(double f) { x *= f; y *= f; }

	// dot product
	double operator%(const Vec_t& rhs) const { return x * rhs.x + y * rhs.y; }

	// unary minus
	Vec_t operator-() const { return Vec_t(-x, -y); }

	// Length() - Returns the length of the vector.
	double Length() const { return sqrt(x * x + y * y); }

	// AngleWith() - Returns the angle which this vector makes with the
	// provided vector, in radians. The value is always positive, in the
	// range 0 to pi.
	double AngleWith(const Vec_t& rhs) const 
	{ 
		return acos(((*this) % rhs) / (Length() * rhs.Length()));
	}

	// Normal() - Returns the normal of the vector, ie a vector perpendicular
	// to it (rotated clockwise by 90 degrees).
	Vec_t Normal() const { return Vec_t(-y, x); }

	// Normalise() - Normalises the vector
	void Normalise()
	{
		double Len = Length();
		x /= Len;
		y /= Len;
	}
};

/*************************************************************************/

#endif // _VEC_H
